package fr.openpeople.test.fibonacci.core.functest;

import static org.junit.Assert.*;

import org.junit.Test;

import static fr.openpeople.test.fibonacci.core.MethodeCalculFibonacci.*;
import fr.openpeople.test.fibonacci.core.SuiteFibonacci;

/**
 * Classe de test (sous JUnit 4) de
 *  fr.openpeople.test.fibonacci.core.SuiteFibonacci.
 * 
 * @author KR
 * @see fr.openpeople.test.fibonacci.core.SuiteFibonacci
 */
public class TestSuiteFibonacci {

	/**
	 * Points de comparaison pour le calcul des valeurs de la suite de Fibonacci
	 *  (ensemble de valeurs connues).
	 */
	public static final int[] valeursRef = {
			// les deux constantes de départ
			0, 1,
			// valeurs connues...
			1, 2, 3, 5, 8, 13, 21, 34, 55, 89,
			144, 233, 377, 610, 987,
			1597, 2584, 4181, 6765,
			10946};

	/**
	 * Méthode de test unitaire JUnit4
	 * pour la méthode <code>calculerFibonacciIter</code>.
	 * @see calculerFibonacciIter
	 */
	@Test
	public void testCalculerFibonacciIter() {
		// tester la méthode itérative (triviale)
		for (int cpt = 0; cpt < valeursRef.length; cpt++) {
			String msgErr = String.format(
					"Problème avec calculerFibonacciIter(%d)",
					cpt);
			long calc =
				(long) SuiteFibonacci.calculerFibonacci(cpt, CALCUL_ITERATIF);
			assertEquals(msgErr, valeursRef[cpt], calc);
		}
		long fibo50 =
			(long) SuiteFibonacci.calculerFibonacci(50, CALCUL_ITERATIF);
		assertEquals("Problème avec calculerFibonacciIter(50)",
				12586269025l,
				fibo50);
		long fibo71 =
			(long) SuiteFibonacci.calculerFibonacci(71, CALCUL_ITERATIF);
		assertEquals("Problème avec calculerFibonacciIter(71)",
				308061521170129l,
				fibo71);
	}

	/**
	 * Méthode de test unitaire JUnit4
	 * pour la méthode <code>calculerFibonacciFonc</code>.
	 * @see calculerFibonacciFonc
	 */
	@Test
	public void testCalculerFibonacciFonc() {
		// tester la méthode fonctionnelle (plus efficace)
		for (int cpt = 0; cpt < valeursRef.length; cpt++) {
			String msgErr = String.format(
					"Problème avec calculerFibonacciFonc(%d)",
					cpt);
			long calc = Math.round(SuiteFibonacci.calculerFibonacci(
					cpt,
					CALCUL_FONCTIONNEL));
			assertEquals(msgErr, valeursRef[cpt], calc);
		}
		long fibo50 = Math.round(SuiteFibonacci.calculerFibonacci(
				50,
				CALCUL_FONCTIONNEL));
		assertEquals("Problème avec calculerFibonacciFonc(50)",
				12586269025l,
				fibo50);
		long fibo71 = Math.round(SuiteFibonacci.calculerFibonacci(
				71,
				CALCUL_FONCTIONNEL));
		assertEquals("Problème avec calculerFibonacciFonc(71)",
				308061521170130l,
				fibo71);
		// il y a une imprécision liée au nombre de chiffres significatifs
		//  disponibles avec un flottant pour calculerFiboFonc(n > 70)...
		//  En revanche, cette fonction peut aller au delà de n = 92,
		//  contrairement à la version itérative aux résultats exacts.
	}

	/**
	 * Méthode s'assurant que la méthode <code>calculerFibonacciIter</code>
	 * renvoie bien une exception lorsqu'on lui passe un argument trop élevé
	 * (<code>n</code> > 92).
	 */
	@Test(expected=IllegalArgumentException.class)
	public void testExceptionFibonacciIter() {
		SuiteFibonacci.calculerFibonacci(93, CALCUL_ITERATIF);
		// inutile de chercher à récupérer un résultat :
		//  une exception doit être lancée...
	}

}
