package fr.openpeople.test.fibonacci.view;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import fr.openpeople.test.fibonacci.core.MethodeCalculFibonacci;
import fr.openpeople.test.fibonacci.core.SuiteFibonacci;
import static fr.openpeople.test.fibonacci.core.MethodeCalculFibonacci.*;

public class FibonacciViewPart extends ViewPart {

	/// CONSTANTS ///

	private static final String CALCULATION_METHOD_GROUP_TITLE =
		"Methode de calcul:";
	private static final String ITERATIVE_METHOD_RADIO_LABEL =
		"iterative";
	private static final String FUNCTIONAL_METHOD_RADIO_LABEL =
		"fonctionnelle";
	private static final String INPUT_VALUE_LABEL =
		"Entrez le parametre pour lequel vous voulez" +
		" calculer la valeur de la suite de Fibonacci :";
	private static final String CALCULATE_BUTTON_LABEL =
		"Calculer !";
	private static final String RESULTS_GROUP_TITLE =
		"Resultats:";
	private static final String RESULT_FORMAT_STRING_ITER =
		"Fibo(%d) = %d";
	private static final String RESULT_FORMAT_STRING_FUNC =
		"Fibo(%d) = %.0f";
	private static final String FUNC_METHOD_NOTE =
		"\n NOTE: la methode de calcul fonctionnelle peut produire" +
		" des resultats imprecis et/ou inexacts pour des valeurs" +
		" elevees (>70).";

	private static final int CONTROLS_OFFSET = 8;

	/// U.I. CONTROLS ///

	Button iterMethodRbtn;
	Button funcMethodRbtn;
	Spinner inputValueSpinner;
	Button calcBtn;
	Text resultsText;

	/// INTERNAL UTILITY METHOD ///

	private void calculateFibonacciSuite() {
		MethodeCalculFibonacci method = CALCUL_ITERATIF;
		if (this.funcMethodRbtn.getSelection())
			method = CALCUL_FONCTIONNEL;
		int param = this.inputValueSpinner.getSelection();
		try {
			if (method == CALCUL_ITERATIF) {
				long res =
					Math.round(SuiteFibonacci.calculerFibonacci(param, method));
				this.resultsText.setText(String.format(
						RESULT_FORMAT_STRING_ITER, param, res));
			} else if (method == CALCUL_FONCTIONNEL) {
				double res = SuiteFibonacci.calculerFibonacci(param, method);
				this.resultsText.setText(String.format(
						RESULT_FORMAT_STRING_FUNC, param, res));
				this.resultsText.append(FUNC_METHOD_NOTE);
			}
		} catch (Exception exc) {
			MessageBox mb = new MessageBox(getSite().getShell(),
					SWT.ICON_WARNING);
			mb.setMessage(exc.getLocalizedMessage());
			mb.setText(exc.getClass().getName());
			mb.open();
			return;
		}
	}

	/// CONSTRUCTOR //

	/**
	 * Default (and unique) constructor.
	 */
	public FibonacciViewPart() {
		// nothing to do here...
	}

	/// INHERITED PUBLIC METHODS ///

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		parent.setLayout(new GridLayout(2, false));
		GridData gd;

		Group calcMethodGroup = new Group(parent, SWT.BORDER);
		calcMethodGroup.setText(CALCULATION_METHOD_GROUP_TITLE);
		{
			RowLayout rl = new RowLayout(SWT.HORIZONTAL);
			rl.center = rl.fill = rl.justify = true;
			rl.marginHeight = rl.marginWidth = CONTROLS_OFFSET;
			rl.pack = false;
			rl.spacing = CONTROLS_OFFSET;
			rl.wrap = false;
			calcMethodGroup.setLayout(rl);

			this.iterMethodRbtn = new Button(calcMethodGroup, SWT.RADIO);
			this.iterMethodRbtn.setText(ITERATIVE_METHOD_RADIO_LABEL);

			this.funcMethodRbtn = new Button(calcMethodGroup, SWT.RADIO);
			this.funcMethodRbtn.setText(FUNCTIONAL_METHOD_RADIO_LABEL);

			this.iterMethodRbtn.setSelection(true);
		}
		gd = new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1);
		calcMethodGroup.setLayoutData(gd);

		Label inputValueLbl = new Label(parent, SWT.NONE);
		inputValueLbl.setText(INPUT_VALUE_LABEL);
		gd = new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1);
		inputValueLbl.setLayoutData(gd);

		this.inputValueSpinner = new Spinner(parent, SWT.NONE);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		this.inputValueSpinner.setLayoutData(gd);

		this.calcBtn = new Button(parent, SWT.PUSH);
		this.calcBtn.setText(CALCULATE_BUTTON_LABEL);
		gd = new GridData(SWT.CENTER, SWT.CENTER, false, false);
		this.calcBtn.setLayoutData(gd);
		this.calcBtn.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				calculateFibonacciSuite();
			}
		});

		Group resultsGroup = new Group(parent, SWT.BORDER);
		resultsGroup.setText(RESULTS_GROUP_TITLE);
		{
			FillLayout fl = new FillLayout();
			fl.marginHeight = fl.marginWidth = CONTROLS_OFFSET;
			resultsGroup.setLayout(fl);

			this.resultsText = new Text(resultsGroup,
					SWT.BORDER | SWT.V_SCROLL |
					SWT.MULTI | SWT.WRAP | SWT.READ_ONLY);
			this.resultsText.setText("");
		}
		gd = new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
		resultsGroup.setLayoutData(gd);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		this.calcBtn.setFocus();
	}

}
